package com.itko.lisa.ext.trilead;

import com.itko.lisa.editor.CustomEditor;
import com.itko.lisa.editor.NodeListDropdown;
import com.itko.lisa.editor.TestCaseInfo;
import com.itko.lisa.editor.TestNodeInfo;
import com.itko.lisa.ext.trilead.xterm.*;
import com.itko.lisa.gui.LisaFrame;
import com.itko.lisa.gui.LisaGuiPanel;
import com.itko.lisa.gui.LisaPrefs;
import com.itko.lisa.gui.ViewResultMasterPanel;
import com.itko.lisa.test.TestExec;
import com.itko.lisa.test.TestRunException;
import com.itko.util.StrUtil;
import com.itko.util.swing.Borders;
//import com.itko.util.swing.GridPanel;
import com.itko.util.swing.MutableGridPanel;
import com.itko.util.swing.PrefillCombo;
import com.itko.util.swing.panels.ProcessingCallback;
import com.itko.util.swing.panels.ProcessingDialogCancelledException;
import com.trilead.ssh2.Connection;
import com.trilead.ssh2.Session;

import java.awt.*;

import javax.swing.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class SSHStepEditor extends CustomEditor {

	private static final long serialVersionUID = -580321237042596823L;
	
	static protected Log log = LogFactory.getLog("com.itko.lisa.ext.ssh");
	
	public static int EXEC_TEST_CONN_ONLY = 1;
	public static int EXEC_SSH_CMD = 2;
	public static int EXEC_LAUNCH_TERM = 3;
	public static int EXEC_TERM_CMD = 4;
	
    private JPanel editor;
    private ViewResultMasterPanel results;

    private PrefillCombo usernameCB;
    private JPasswordField passwordCB;
    private PrefillCombo hostCB;
    private PrefillCombo portCB;

    private JComboBox authCB;
    private PrefillCombo privateKeyCB;
    private JPasswordField privateKeyPassCB;

    private JComboBox modeCB;
    private PrefillCombo nonIntCommandCB;
    private PrefillCombo intExpectCB;
    private PrefillCombo intCommandCB;
    private PrefillCombo intExpectAfterCB;
    private NodeListDropdown onError;
    private JButton execB;
    private JButton launchTermB;
    private JButton execTermB;
    private XtermUI lisaTerminal;
    
    private Connection conn;

    private void setup()
    {
        setupEditor();
        setupResponseEditor();

        setLayout( new BorderLayout() );
        add( editor, BorderLayout.NORTH );
        add( results, BorderLayout.CENTER );
    }


    private void setupEditor()
    {
        usernameCB = new PrefillCombo(getClass().getName() + ".user", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        passwordCB = new JPasswordField();
        hostCB = new PrefillCombo(getClass().getName() + ".host", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        portCB = new PrefillCombo(getClass().getName() + ".port", null, null, LisaPrefs.getLisaPrefsObj(), true, this );

        privateKeyCB = new PrefillCombo(getClass().getName() + ".private", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        privateKeyCB.setEnabled(false);
        privateKeyPassCB = new JPasswordField();
        privateKeyPassCB.setEnabled(false);
        
        authCB = new JComboBox(TrileadConnectionFactory.AUTH_TYPES);
        authCB.setEditable(false);
        authCB.addActionListener( new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		if (authCB.getSelectedItem().equals(TrileadConnectionFactory.AUTH_PUBLIC_KEY)) {
        			privateKeyCB.setEnabled(true);
        			privateKeyPassCB.setEnabled(true);
        		} else {
        			privateKeyCB.setEnabled(false);
        			privateKeyPassCB.setEnabled(false);
        		}
        	}
        });
        
        modeCB = new JComboBox(SSHStep.MODE_ARR);
        modeCB.setEditable(false);
        modeCB.setEnabled(true);
        modeCB.addActionListener( new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		if (modeCB.getSelectedItem().equals(SSHStep.INTERACTIVE_MODE)) {
        			nonIntCommandCB.setEnabled(false);
        			intCommandCB.setEnabled(true);
        			intExpectCB.setEnabled(true);
        			execB.setEnabled(false);
        			intExpectAfterCB.setEnabled(true);
        			launchTermB.setEnabled(true);
        			execTermB.setEnabled(true);
        		} else {
        			nonIntCommandCB.setEnabled(true);
        			intCommandCB.setEnabled(false);
        			intExpectCB.setEnabled(false);
        			execB.setEnabled(true);
        			intExpectAfterCB.setEnabled(false);
        			launchTermB.setEnabled(false);
        			execTermB.setEnabled(false);
        		}
        	}
        });
        nonIntCommandCB = new PrefillCombo(getClass().getName() + ".command", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        intExpectCB = new PrefillCombo(getClass().getName() + ".expect", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        intExpectCB.setEnabled(false);
        intCommandCB = new PrefillCombo(getClass().getName() + ".intCommand", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        intCommandCB.setEnabled(false);
        intExpectAfterCB = new PrefillCombo(getClass().getName() + ".expectAfter", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        intExpectAfterCB.setEnabled(false);
        onError = new NodeListDropdown( false, true );
        onError.setAllowEmpty( false );
        onError.setAllowProperty( true );
        
        JButton testConnB = new JButton("Test Connection");
        testConnB.addActionListener( new ActionListener() {
        	public void actionPerformed(ActionEvent event) {
        		
        		// Just test the connection only
        		testButton(EXEC_TEST_CONN_ONLY);
        	}
        });
        JPanel testConnHolder = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
        testConnHolder.add( testConnB );
        
        // Create the execute test button
        execB = new JButton( "Test SSH Command" );
        execB.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent event ) {
            	
            	// Run the full execute
            	testButton(EXEC_SSH_CMD);
            }
        });
        JPanel testExecHolder = new JPanel( new FlowLayout(FlowLayout.LEFT) );
        testExecHolder.add(execB);
//        execHolder.add( executeB );
        
        // Create the launch terminal button
//        TestNodeInfo ni = (TestNodeInfo)getController();
//        SSHStep sshNode = (SSHStep)ni.getAttribute( SSHStepController.SSH_STEP_KEY );
//		lisaTerminal = new XtermUI(LisaFrame.getMe(), sshNode);
        lisaTerminal = new XtermUI(LisaFrame.getMe());
        launchTermB = new JButton("Launch Terminal");
        launchTermB.addActionListener( new ActionListener() {
        	public void actionPerformed( ActionEvent event ) {
        		
        		// Run the terminal execute
        		testButton(EXEC_LAUNCH_TERM);
        	}
        });
        JPanel termButtonHolder = new JPanel( new FlowLayout(FlowLayout.LEFT) );
        termButtonHolder.add( launchTermB );
        
        // Create the execute terminal command test button
        execTermB = new JButton("Test Terminal Command");
        execTermB.addActionListener( new ActionListener() {
        	public void actionPerformed(ActionEvent event) {
        		testButton(EXEC_TERM_CMD);
        	}
        });
//        termButtonHolder.add( execTermB );

        MutableGridPanel gpConnect = new MutableGridPanel(9,5);
        gpConnect.add("Username: ", usernameCB.getComponent());
        gpConnect.add("Password: ", passwordCB);
        gpConnect.add("Host: ", hostCB.getComponent());
        gpConnect.add("Port: ", portCB.getComponent());
        
        gpConnect.add("Authentication Scheme", authCB);
        gpConnect.add("Private Key", privateKeyCB.getComponent());
        gpConnect.add("Private Key Passphrase", privateKeyPassCB);
        gpConnect.add("On Error: ", onError);
        gpConnect.add(testConnHolder);
        JPanel connectPanel = gpConnect.getPanel();
        connectPanel.setBorder( Borders.createGroupBox("Connection Information"));
        
        //GridPanel gpData = new GridPanel(6);
        MutableGridPanel gpData = new MutableGridPanel(6,5);

        gpData.add("Shell Mode", modeCB);
        gpData.add("SSH Command: ", nonIntCommandCB.getComponent());
        gpData.add(testExecHolder);
        gpData.add("Expect Before:", intExpectCB.getComponent());
        gpData.add("Interactive Command:", intCommandCB.getComponent());
//        gpData.add("Expect After:", intExpectAfterCB.getComponent());
        gpData.add(termButtonHolder);
        JPanel dataPanel = gpData.getPanel();
        dataPanel.setBorder( Borders.createGroupBox("Command Information") );
        
//        editor = new JPanel( new BorderLayout());
        editor = new JPanel( new GridLayout(1,2) );
        editor.setBorder( Borders.createGroupBox(null));
        editor.add(connectPanel);
        editor.add(dataPanel);
    }

    private void setupResponseEditor()
    {
        results = new ViewResultMasterPanel( false, true );
    }
    
    private boolean makeConnection(String tsHostname, int tsPort, String tsUsername, String tsPassword, String tsAuth, String tsPrivateKey, String tsPrivateKeyPass) {
    	
    	boolean bPassed = false;
    	try {
    		conn = TrileadConnectionFactory.getConnection(tsHostname, tsPort, tsUsername, tsPassword, tsAuth, tsPrivateKey, tsPrivateKeyPass);
    		bPassed = TrileadConnectionFactory.testConnection(conn);
    	} catch (Exception e) {
    		// eat
    	}
    	
    	return bPassed;
    }
    
    private void testButton(final int execMode)
    {
        final TestNodeInfo ni = (TestNodeInfo)getController();
        final TestCaseInfo tci = ni.getTestCaseInfo();
        final TestExec ts = tci.getTestExec();
        
        final String sshErrorNode = onError.getCurrentNodeName();
        final String sshMode = modeCB.getSelectedItem().toString();

        final String sshHostname = hostCB.getText();
        final String tsHostname = ts.parseInState(sshHostname);
        final String sshPort = portCB.getText();
        final int tsPort = Integer.parseInt(sshPort);
        final String sshUsername = usernameCB.getText();
        final String tsUsername = ts.parseInState(sshUsername);
        final String sshPassword = new String(passwordCB.getPassword());
        final String tsPassword = ts.parseInState(sshPassword);
        final String sshCommand = nonIntCommandCB.getText();
        final String tsCommand = ts.parseInState(sshCommand);
        final String sshAuth = authCB.getSelectedItem().toString();
        final String tsAuth = ts.parseInState(sshAuth);
        final String sshPrivateKey = privateKeyCB.getText();
        final String tsPrivateKey = ts.parseInState(sshPrivateKey);
        final String sshPrivateKeyPass = new String(privateKeyPassCB.getPassword());
        final String tsPrivateKeyPass = ts.parseInState(sshPrivateKeyPass);
        
        String msg = isEditorValid();
        if( msg != null ) {
            LisaGuiPanel.infoMsg( this, getName(), msg );
            return;
        }
        
        final StringBuffer sb = new StringBuffer();
        try {
            LisaGuiPanel.doProcessingDialog( getName(), this,
                "Doing SSH",
                new ProcessingCallback() {
                    public void doCallback( Object o ) throws Throwable
                    {
                    	boolean bPassed = makeConnection(tsHostname, tsPort, tsUsername, tsPassword, tsAuth, tsPrivateKey, tsPrivateKeyPass);
                    	
                    	if (execMode == EXEC_SSH_CMD) {
                    		// Get the connection and session
                    		Session sess = conn.openSession();
                    		
                    		// Run the command
                			SSHResult result = SSHStep.execSSHCommand(sess, tsCommand);
                			
                			// Check the return code
                			String msg = result.getStdout();
                			if (result.getExitStatus() != null) {
                				if (result.getExitStatus().intValue() != 0) {
                					msg = result.getStderr();
                    				sb.append("SSH command completed. Exit code = " + result.getExitStatus());
                				}
                				ts.setStateValue("ssh.last.exitstatus", result.getExitStatus());
                			} else {
                				sb.append("SSH command not completed.  No exit code returned.");
                			}
	                    	ni.setRet(msg);
	                    	
	                    	// Cleanup
                    		sess.close();
                    		TrileadConnectionFactory.cleanupConnection(conn, tsHostname, tsUsername, tsAuth, false);                    		
                    	} else if (execMode == EXEC_TEST_CONN_ONLY) {
                    		// Just test the connection only
                    		if (bPassed) {
                    			sb.append("Connection Test Passed.");
                    			TrileadConnectionFactory.cleanupConnection(conn, tsHostname, tsUsername, tsAuth, false);
                    		} else {
                    			sb.append("Connection Test Failed.");
                    			TrileadConnectionFactory.cleanupConnection(conn, tsHostname, tsUsername, tsAuth, true);
                    		}
                        } else if (execMode == EXEC_LAUNCH_TERM) {
                    		// Display the LISA Terminal
                    		lisaTerminal.loadEngine(tsHostname, tsUsername, tsAuth, conn.openSession());
                    		
                    		// Update the settings on the terminal from the current step
                    		lisaTerminal.setSshAuth(sshAuth);
                    		lisaTerminal.setSshErrorNode(sshErrorNode);
                    		lisaTerminal.setSshHostname(sshHostname);
                    		lisaTerminal.setSshMode(sshMode);
                    		lisaTerminal.setSshPassword(sshPassword);
                    		lisaTerminal.setSshPrivateKey(sshPrivateKey);
                    		lisaTerminal.setSshPrivateKeyPassword(sshPrivateKeyPass);
                    		lisaTerminal.setSshUsername(sshUsername);
                    		
                    		lisaTerminal.startEngine(XtermEngine.STATUS_RECORDING, tci);
                    		lisaTerminal.setVisible(true);
                    		return;
                        } else if (execMode == EXEC_TERM_CMD) {
                        	// Load up the LISA Terminal
                        	lisaTerminal.loadEngine(tsHostname, tsUsername, tsAuth, conn.openSession());
                        	
                        	// Update the settings on the terminal from the current step
                    		lisaTerminal.setSshAuth(sshAuth);
                    		lisaTerminal.setSshErrorNode(sshErrorNode);
                    		lisaTerminal.setSshHostname(sshHostname);
                    		lisaTerminal.setSshMode(sshMode);
                    		lisaTerminal.setSshPassword(sshPassword);
                    		lisaTerminal.setSshPrivateKey(sshPrivateKey);
                    		lisaTerminal.setSshPrivateKeyPassword(sshPrivateKeyPass);
                    		lisaTerminal.setSshUsername(sshUsername);
                    		
                        	lisaTerminal.startEngine(XtermEngine.STATUS_PLAYBACK, tci);
                        	lisaTerminal.setVisible(true);
                        	
                        	// TODO: Actually dump the command on it
                        	
                    	} else {
                    		sb.append("Unknown execution mode: " + execMode);
                    	}
                    }
                }, true );
            
            // Figure out whether or not to popup a message
            if (sb.length() != 0) {
            	// Popup dialog with response
            	LisaGuiPanel.infoMsg( this, getName(), sb.toString() );
            }
        }
        catch( ProcessingDialogCancelledException can )
        {
            log.info( "You gave up, but the SSH will continue until it completes in the background." );
        }
        catch( Throwable tw )
        {
        	log.debug("Test button threw a Throwable:", tw);
            TestRunException tre = new TestRunException("Error executing SSH: " + " " + tw.getMessage(), tw);
            LisaGuiPanel.errorMsg( this, getName(), "Error executing SSH: " + " " + tw.getMessage(), tre );
        }

        results.setNodeInfo( tci, ni );
    }
    
    public void display()
    {
        if( usernameCB == null )
            setup();

        TestNodeInfo ni = (TestNodeInfo)getController();
        SSHStep ssh = (SSHStep)ni.getAttribute( SSHStepController.SSH_STEP_KEY );
        usernameCB.setText( ssh.getUsername() );
        passwordCB.setText( ssh.getPassword() );
        hostCB.setText( ssh.getHostname() );
        portCB.setText( ssh.getPort() );
        if (ssh.getAuth() == null) {
        	authCB.setSelectedItem(TrileadConnectionFactory.AUTH_PASSWORD);
        } else {
        	authCB.setSelectedItem( ssh.getAuth() );
        }
        privateKeyCB.setText( ssh.getPrivateKey() );
        privateKeyPassCB.setText( ssh.getPrivateKeyPass() );

        nonIntCommandCB.setText( ssh.getCommand() );
        intCommandCB.setText( ssh.getIntCommand() );
        intExpectCB.setText( ssh.getIntExpect() );
        if (ssh.getMode() == null) {
        	modeCB.setSelectedItem(SSHStep.NON_INTERACTIVE_MODE);
        } else {
        	modeCB.setSelectedItem( ssh.getMode() );
        }
        
        onError.setList( ni.getTestCaseInfo().getNodes() );
        if (!StrUtil.isEmpty(ssh.getErrorNode())) {
            onError.setCurrentNodeName( ssh.getErrorNode() );
        } else {
            onError.setCurrentNodeName( "fail" );
        }
        
        results.setNodeInfo( ni.getTestCaseInfo(), ni );
    }


    public String isEditorValid()
    {
        if( StrUtil.isEmpty( hostCB.getText() ) )
            return "Need a host";
        
        if( onError.getCurrentNode() == null )
            return "Please provide an On Error Node";

        return null;
    }


    public void save()
    {
        TestNodeInfo ni = (TestNodeInfo)getController();
        ni.getTestCaseInfo().getTestExec().saveNodeResponse( ni.getName(), ni.getRet() );
        SSHStep ssh = (SSHStep)ni.getAttribute( SSHStepController.SSH_STEP_KEY );
        ssh.setUsername( usernameCB.getText() );
        ssh.setPassword( new String(passwordCB.getPassword()));
        ssh.setHostname( hostCB.getText() );
        ssh.setPort( portCB.getText() );
        ssh.setAuth( authCB.getSelectedItem().toString() );
        ssh.setPrivateKey( privateKeyCB.getText() );
        ssh.setPrivateKeyPass( new String(privateKeyPassCB.getPassword()) );
        
        ssh.setCommand( nonIntCommandCB.getText() );
        ssh.setIntCommand( intCommandCB.getText() );
        ssh.setIntExpect( intExpectCB.getText() );
        ssh.setMode( modeCB.getSelectedItem().toString() );
        ssh.setErrorNode( onError.getCurrentNodeName() );
    }
}
